-------------------------------------------------------------------------------
--                G  O  B  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

-- Labels for the key bindings
_G["BINDING_HEADER_NIGGLES"]                     = L["Niggles"];
_G["BINDING_NAME_CLICK NigglesMount:LeftButton"] = L["Mount"];
_G["BINDING_NAME_NIGGLES_TOGGLELOCATOR"]         = L["Toggle Locator"];

-- Account data saved for the Niggles addon
NigglesDB =
{
  raidAssists = {},
  moderators  = {},
  nonRaiders  = {}
};

-- Character data saved for the Niggles addon
NigglesDBC =
{
  faction  = {},
  settings = {},
  mounts   = {},
  critters = {}
};

-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local HELP_COLOR_CODE = "|cFF8080FF";

-- Item IDs for all know Argent Tournament lances
local tournamentLances =
{
  [46069] = true, -- Alliance Lance
  [46070] = true, -- Horde Lance
  [46106] = true, -- Argent Lance
  [52716] = true  -- Twilight Firelance
};

-- Default values for all Niggles settings
local defaults =
{
  -- General Options
  ["UiAutoLootArch"]      = false,
  ["UiAutoLootFish"]      = false,
  ["UiAutoLootMoney"]     = false,
  ["UiChatClassColor"]    = true,
  ["UiEMIgnoreToggle"]    = true,
  ["UiFirstLastPage"]     = true,
  ["UiFlightList"]        = true,
  ["UiGuildRosterFilter"] = true,
  ["UiItemDelete"]        = true,
  ["UiMailFont"]          = true,
  ["UiMapLocation"]       = true,
  ["UiMerchantSearch"]    = true,
  ["UiMiniMapZoom"]       = true,
  ["UiQuestRewards"]      = true,
  ["UiRestoreFaction"]    = true,

  -- Warning Options
  ["WarningCurrency"]     = true,
  ["WarningCurrencyMask"] = 0x02,
  ["WarningRod"]          = true,
  ["WarningLance"]        = true,
  ["WarningNoAspect"]     = false,
  ["WarningPetGrowl"]     = false,
  ["LogChat"]             = false,
  ["LogCombat"]           = false,

  -- Profession Options
  ["ProfArchHilite"]      = true,
  ["ProfArchKeystones"]   = true,
  ["ProfArchEnhIcon"]     = true,
  ["ProfArchRClickBack"]  = true,

  -- Raids Options
  ["RaidsTarget"]         = true,
  ["RaidsAutoPromote"]    = true,
  ["RaidsAutoLootMethod"] = true,
  ["RaidsLootMethod"]     = "master",
  ["RaidsAutoLootThold"]  = true,
  ["RaidsLootThreshold"]  = 4,

  -- Calendar Event Options
  ["CalEventsList"]       = true,
  ["CalEventsInvites"]    = true,
  ["CalEventsOnline"]     = true,
  ["CalEventsFilter"]     = false,
  ["CalEventsInactive"]   = 60,

  -- Companion Options
  ["MountsEnabled"]       = true,
  ["MountsClassRace"]     = true,
  ["MountsHoliday"]       = false,
  ["CrittersEnabled"]     = true,
  ["CrittersInKalimdor"]  = true,
  ["CrittersInEKingdoms"] = true,
  ["CrittersInNorthrend"] = true,
  ["CrittersInOutland"]   = true,
  ["CrittersInCities"]    = true,
  ["CrittersInDungeons"]  = true,
  ["CrittersInRaids"]     = false,
  ["CrittersInArenas"]    = false,
  ["CrittersInBG"]        = false,

  -- Locator Options
  ["LocatorShown"]        = false
};

-- Valid commands for Niggles
local validCommands =
{
  {command = L["General"]:lower(),     category = "optionsGeneral"    },
  {command = L["Warnings"]:lower(),    category = "optionsWarnings"   },
  {command = L["Professions"]:lower(), category = "optionsProfessions"},
  {command = L["Raids"]:lower(),       category = "optionsRaids"      },
  {command = L["Events"]:lower(),      category = "optionsCalEvents"  },
  {command = L["Mounts"]:lower(),      category = "optionsCompanions" },
  {command = L["Companions"]:lower(),  category = "optionsCompanions" }
};

-------------------------------------------------------------------------------
--                   S  L  A  S  H     C  O  M  M  A  N  D  S
-------------------------------------------------------------------------------

SLASH_NIGGLES1 = "/niggles";
SLASH_NIGGLES2 = "/ngl";

-------------------------------------------------------------------------------
--                 L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to make a simple copy of a table
--
local function tableCopy(table)
  local copy = {};
  for key, value in pairs(table) do
    copy[key] = value;
  end
  return copy;
end

--
-- Function to migrate data used by the addon from old to new formats
--
local function migrateData()
  -- Migrate any old data to the new format for the mount list
  if (NigglesDBC.mounts["run"] ~= nil) then
    for spellId, _ in pairs(NigglesDBC.mounts["run"]) do
      NigglesDBC.mounts[spellId] =
        bit.bor((NigglesDBC.mounts[spellId] or 0), 0x01);
    end
    NigglesDBC.mounts["run"] = nil;
  end
  if (NigglesDBC.mounts["fly"] ~= nil) then
    for spellId, _ in pairs(NigglesDBC.mounts["fly"]) do
      NigglesDBC.mounts[spellId] =
        bit.bor((NigglesDBC.mounts[spellId] or 0), 0x02);
    end
    NigglesDBC.mounts["fly"] = nil;
  end
  if (NigglesDBC.mounts["swim"] ~= nil) then
    for spellId, _ in pairs(NigglesDBC.mounts["swim"]) do
      NigglesDBC.mounts[spellId] =
        bit.bor((NigglesDBC.mounts[spellId] or 0), 0x08);
    end
    NigglesDBC.mounts["swim"] = nil;
  end

  return;
end

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to process the slash command
--
function SlashCmdList.NIGGLES(message, editbox)
  -- Local Variables
  local command, arguments = message:match("^(%S*)%s*(.-)$");
  local category = nil

  -- Check a command was specified
  if ((command ~= nil) and (command ~= "")) then
    -- Search the valid commands for one that matches the specified command
    for _, cmdInfo in ipairs(validCommands) do
      if (cmdInfo.command:find("^"..command)) then
        category = cmdInfo.category;
        break;
      end
    end
  end

  -- Check if the command was found
  if (category ~= nil) then
    InterfaceOptionsFrame_OpenToCategory(Niggles[category]);
  else
    -- Display the help
    DEFAULT_CHAT_FRAME:AddMessage(HELP_COLOR_CODE.."/niggles||/ngl");
    for _, cmdInfo in ipairs(validCommands) do
      DEFAULT_CHAT_FRAME:AddMessage(HELP_COLOR_CODE..
        string.gsub(cmdInfo.command, "(.)(.*)", "  %1[%2]"));
    end
  end

  return;
end

--
-- Function to initialise the addon
--
function NigglesOnLoad(self)
  -- Initialise some variables
  self.updatedOptions = nil;
  self.lastOnline     = {};
  self.critterIdx     = 0;
  self.critterMask    = 0;
  self.mainHandSlotId = GetInventorySlotInfo("MainHandSlot");
  self.updateRaid     = false;
  self.playerZoneId   = -1;
  self.timers         =
  {
    elapsed = 0,
    list = nil
  };

  -- Get the player's class
  _, self.playerClass = UnitClass("player");
  _, self.playerRace  = UnitRace("player");

  -- Hook the standard UI
  NigglesHookUi();

  -- Initialise the changes to the Spell Book panel
  NigglesSpellBookInit();

  -- Initialise the changes to the Quest panels
  NigglesQuestsInit();

  -- Register for some events
  self:RegisterEvent("ADDON_LOADED");
  self:RegisterEvent("PLAYER_LOGIN");
  self:RegisterEvent("SPELLS_CHANGED");
  self:RegisterEvent("LEARNED_SPELL_IN_TAB");
  self:RegisterEvent("GET_ITEM_INFO_RECEIVED");
  self:RegisterEvent("ZONE_CHANGED_NEW_AREA");
  self:RegisterEvent("ZONE_CHANGED_INDOORS");

  return;
end

--
-- Function to process events
--
function NigglesOnEvent(self, event, ...)
  -- Process the event that occurred
  if (event == "COMBAT_LOG_EVENT_UNFILTERED") then
    -- Check a spell has been successfully cast
    local _, eventType = ...;
    if (eventType == "SPELL_CAST_SUCCESS") then
      -- Check the spell is 'Growl' from a friendly pet
      local _, _, _, sourceGUID, sourceName, sourceFlags, _, _, _, _, _,
        _, spellName = ...;

      -- Check the event is from a friendly pet
      if ((spellName == "Growl") and
          (tonumber(sourceGUID:sub(5,5), 16)%8 == 4) and
          (bit.bor(sourceFlags, COMBATLOG_OBJECT_REACTION_FRIENDLY) ~= 0)) then
        -- Get the type of instance the player is in
        local _, instanceType = IsInInstance();

        -- Check if the player is in a party or raid instance
        if (((instanceType == "party") or
             (instanceType == "raid")) and
             (GetNumPartyMembers() > 0)) then
          Niggles:AddMessage(format(L["Pet Growled"], sourceName),
            1.0, 1.0, 0.1, 1.0, 3);
          PlaySound("RaidWarning");
        end
      end
    end
  elseif (event == "PLAYER_REGEN_DISABLED") then
    -- Check if the player is in a vehicle
    if (not UnitInVehicle("player")) then
      -- Local Variables
      local playWarning = false;

      -- Get the ID of the weapon in the player's main hand
      local itemId = GetInventoryItemID("player", self.mainHandSlotId);
      if (itemId) then
        -- Display a warning if a lance or fishing rod is equipped
        if ((NigglesDBC.settings.WarningLance) and
            (tournamentLances[itemId])) then
          Niggles:AddMessage(format(L["Equipped"], L["Lance"]),
            1.0, 1.0, 0.1, 1.0, 3);
          playWarning = true;
        elseif ((NigglesDBC.settings.WarningRod) and
                (IsEquippedItemType(L["Fishing Pole"]))) then
          Niggles:AddMessage(format(L["Equipped"], L["Fishing Pole"]),
            1.0, 1.0, 0.1, 1.0, 3);
          playWarning = true;
        end
      end

      -- Display a warning if there is no active aspect
      if ((Niggles.playerClass == "HUNTER") and
          (NigglesDBC.settings.WarningNoAspect) and
          (GetNumShapeshiftForms() > 0) and
          (GetShapeshiftForm() == 0)) then
        Niggles:AddMessage(L["No Aspect"], 1.0, 1.0, 0.1, 1.0, 3);
        playWarning = true;
      end

      -- Play the warning sound, if required
      if (playWarning == true) then
          PlaySound("RaidWarning");
      end
    end
  elseif (event == "PLAYER_REGEN_ENABLED") then
    if (Niggles.updateRaid == true) then
      Niggles.updateRaid = false;
      NigglesRaidButtonsSetTargets();
    end
  elseif ((event == "PARTY_MEMBERS_CHANGED") or
          (event == "RAID_ROSTER_UPDATE")) then
    NigglesRaidButtonsSetTargets();
    if ((NigglesDBC.settings.RaidsAutoPromote == true) and
        (IsRealRaidLeader())) then
      NigglesRaidRosterPromoteAssists();
    end
  elseif (event == "PARTY_CONVERTED_TO_RAID") then
    if (IsRealRaidLeader()) then
      if (NigglesDBC.settings.RaidsAutoLootMethod == true) then
        NigglesTimerCreate("RaidsAutoLootMethod", SetLootMethod,
          NigglesDBC.settings.RaidsLootMethod, "player", 1.0, false);
      end
      if (NigglesDBC.settings.RaidsAutoLootThold == true) then
        NigglesTimerCreate("RaidsAutoLootThold", SetLootThreshold,
          NigglesDBC.settings.RaidsLootThreshold, nil, 2.0, false);
      end
    end
  elseif (event == "BAG_UPDATE") then
    local bagId = select(1, ...);
    if ((bagId >=0) and (bagId <= NUM_BAG_SLOTS)) then
      NigglesCrittersBagUpdate();
    end
  elseif (event == "UPDATE_STEALTH") then
    if (IsStealthed() == 1) then
      DismissCompanion("CRITTER");
      Niggles.critterIdx = 0;
    end
  elseif ((event == "QUEST_DETAIL") or
          (event == "QUEST_PROGRESS") or
          (event == "QUEST_COMPLETE")) then
    NigglesQuestsItemInfoUpdate();
  elseif ((event == "SPELLS_CHANGED") or
          (event == "LEARNED_SPELL_IN_TAB")) then
    NigglesMountsSpellsUpdate();
  elseif (event == "GET_ITEM_INFO_RECEIVED") then
    if ((NigglesDBC.settings.UiQuestRewards) and
        ((QuestFrame:IsShown()) or
         (QuestLogFrame:IsShown()) or
         (QuestLogDetailFrame:IsShown()))) then
      NigglesQuestsItemInfoUpdate();
    end
  elseif (event == "GUILD_ROSTER_UPDATE") then
    NigglesGuildRosterUpdate();
  elseif (event == "CALENDAR_UPDATE_INVITE_LIST") then
    NigglesCalEventInviteListUpdate();
  elseif (event == "CURRENCY_DISPLAY_UPDATE") then
    NigglesCurrencyCheck();
  elseif (event == "ZONE_CHANGED_NEW_AREA") then
    NigglesLocatorZoneChanged();
  elseif (event == "ADDON_LOADED") then
    local addonName = select(1, ...);
    if (addonName == "Blizzard_GuildUI") then
      NigglesGuildRosterInit();
    elseif (addonName == "Blizzard_Calendar") then
      NigglesCalendarInit();
    elseif (addonName == "Blizzard_ArchaeologyUI") then
      NigglesArchaeologyHookPanel();
    end
  elseif (event == "PLAYER_LOGIN") then
    -- Migrate any data from old to new formats
    migrateData();

    -- Initialise the settings
    NigglesSettingsInit();

    -- Initialise the holiday info
    NigglesHolidaysInitialise();

    -- Initialise the Mount info
    NigglesMountsInitialise();
    NigglesMountsSpellsUpdate();
   
    -- Show/Hide the Locator frame
    NigglesLocatorShow(NigglesDBC.settings.LocatorShown);

    -- Register for events
    NigglesRegisterForEvents();
  end

  return;
end

--
-- Function to make sure all the settings are initialised
--
function NigglesSettingsInit()
  -- Make sure all settings are initialised
  if (NigglesDBC.settings == nil) then
    NigglesDBC.settings = {};
  end
  for key, value in pairs(defaults) do
    if (NigglesDBC.settings[key] == nil) then
      NigglesDBC.settings[key] = value;
    end
  end

  -- Make sure the character lists exist
  if (NigglesDB.raidAssists == nil) then
    NigglesDB.raidAssists = {};
  end
  if (NigglesDB.moderators == nil) then
    NigglesDB.moderators = {};
  end
  if (NigglesDB.nonRaiders == nil) then
    NigglesDB.nonRaiders = {};
  end

  -- Make sure the lists of companions exist
  if (NigglesDBC.mounts == nil) then
    NigglesDBC.mounts = {};
  end
  if (NigglesDBC.critters == nil) then
    NigglesDBC.critters = {};
  end

  return;
end

--
-- Function to initialise check buttons in the Niggles Options frame
--
function NigglesOptionsCheckButtonInit(checkButton, setting)
  -- Local Variables
  local textWidth;
  local textFont;

  -- Check if no setting has been specified
  if (setting == nil) then
    -- Work out the button's setting from it's name
    setting = string.gsub(checkButton:GetName(),
      "^"..checkButton:GetParent():GetName().."(.*)$", "%1");
  end

  -- Save the setting the button is for
  checkButton.nglSetting = setting;

  -- Set the check button's font by simulating an enable event
  NigglesOptionsCheckButtonEnabled(checkButton)

  -- Set the check button's text
  checkButton.text:SetText(L[setting]);

  -- Update the size of the button's hit box
  textWidth = checkButton.text:GetStringWidth();
  if ((textWidth ~= nil) and (textWidth > 0)) then
    checkButton:SetHitRectInsets(0, -textWidth, 0, 0);
  end

  return;
end

--
-- Function to refresh settings in Niggles Options panels. This function
-- is recursive.
--
function NigglesOptionsRefreshSettings(widget)
  -- Check if the widget has a Niggles setting associated with it
  if ((widget.nglSetting) and
      (Niggles.updatedOptions.settings[widget.nglSetting] ~= nil)) then
    -- Set the widget's state, based on its type
    local widgetType = widget:GetObjectType()
    if (widgetType == "CheckButton") then
      widget:SetChecked(Niggles.updatedOptions.settings[widget.nglSetting]);
    elseif (widgetType == "EditBox") then
      if (widget:IsNumeric()) then
        widget:SetNumber(Niggles.updatedOptions.settings[widget.nglSetting]);
      else
        widget:SetText(Niggles.updatedOptions.settings[widget.nglSetting]);
      end
      widget:SetCursorPosition(0);
    else
      if (widget.nglObjectType == "UIDropDownMenu") then
        widget:SetValue(Niggles.updatedOptions.settings[widget.nglSetting]);
      end
    end
  end

  -- Process all children of the current widget
  for _, child in ipairs({widget:GetChildren()}) do
    NigglesOptionsRefreshSettings(child);
  end
end

--
-- Function to process check button clicks in the Niggles Options frame
--
function NigglesOptionsCheckButtonClicked(checkButton)
  if (checkButton:GetChecked()) then
    PlaySound("igMainMenuOptionCheckBoxOn");
    Niggles.updatedOptions.settings[checkButton.nglSetting] = true;
  else
    PlaySound("igMainMenuOptionCheckBoxOff");
    Niggles.updatedOptions.settings[checkButton.nglSetting] = false;
  end
end

--
-- Function to process enabling/disabling of a check button
--
function NigglesOptionsCheckButtonEnabled(checkButton)
  -- Local Variables
  local font = "GameFontHighlight";

  -- Work out which font to use
  if (checkButton:IsEnabled()) then
    if (checkButton.normalFont ~= nil) then
      font = checkButton.normalFont;
    end
  else
    if (checkButton.disabledFont ~= nil) then
      font = checkButton.disabledFont;
    else
      font = "GameFontDisable";
    end
  end

  -- Change the check button's font
  checkButton.text:SetFontObject(font);

  return;
end

--
-- Function to process edit box text changes in the Niggles Options frame
--
function NigglesOptionsEditBoxTextChange(editBox)
  if (editBox.nglSetting ~= nil) then
    if (editBox:IsNumeric()) then
      Niggles.updatedOptions.settings[editBox.nglSetting] =
        tonumber(editBox:GetText());
    else
      Niggles.updatedOptions.settings[editBox.nglSetting] = editBox:GetText();
    end
  end
end

--
-- Function to set the tooltip for a widget in the Niggles Options panels
--
function NigglesOptionsCheckButtonSetTooltip(checkButton)
  if (L[checkButton.nglSetting.."Tooltip"] ~= nil) then
    GameTooltip:SetOwner(checkButton, "ANCHOR_TOPLEFT", 20);
    GameTooltip:SetText(L[checkButton.nglSetting.."Tooltip"],
     nil, nil, nil, nil, 1);
    GameTooltip:Show();
  end
end

--
-- Function to set the value of a drop down
--
function NigglesOptionsDropDown_SetValue(self, value)
  -- Save the new value
  Niggles.updatedOptions.settings[self.nglSetting] = value;

  -- Set the drop down menu's value
  UIDropDownMenu_Initialize(self,
    NigglesOptionsRaidsLootMenu_Initialise);
  UIDropDownMenu_SetSelectedValue(self, value);
end

--
-- Function to update the value of a drop down when one of its options is
-- clicked.
--
function NigglesOptionsDropDownOnClick(self, arg1, arg2, checked)
  arg1:SetValue(self.value);
end

--
-- Function to refresh the options for the Niggles Options panels
--
function NigglesOptionsRefresh()
  -- Check there aren't any updated options already
  if (Niggles.updatedOptions == nil) then
    -- Initialise the updated options
    Niggles.updatedOptions = {};

    -- Copy the existing settings
    Niggles.updatedOptions.settings = tableCopy(NigglesDBC.settings);

    -- Copy the existing character lists
    Niggles.updatedOptions.raidAssists = tableCopy(NigglesDB.raidAssists);
    Niggles.updatedOptions.moderators  = tableCopy(NigglesDB.moderators);
    Niggles.updatedOptions.nonRaiders  = tableCopy(NigglesDB.nonRaiders);

    -- Copy the existing lists of companions
    Niggles.updatedOptions.mounts   = tableCopy(NigglesDBC.mounts);
    Niggles.updatedOptions.critters = tableCopy(NigglesDBC.critters);

    -- Initialise all the widgets to the correct state for the settings
    NigglesOptionsRefreshSettings(NigglesOptionsGeneral);
    NigglesOptionsRefreshSettings(NigglesOptionsWarnings);
    NigglesOptionsWarningsEnableFrames();
    NigglesOptionsRefreshSettings(NigglesOptionsProfessions);
    NigglesOptionsRefreshRaids();
    NigglesOptionsRefreshCalEvents();
    NigglesOptionsRefreshCompanions();
  end

  return;
end

--
-- Function to save updated options for the Niggles Options panels
--
function NigglesOptionsSave()
  -- Check if there are any updated options
  if (Niggles.updatedOptions ~= nil) then
    -- Save the new settings
    NigglesDBC.settings = Niggles.updatedOptions.settings;

    -- Save the new lists of companions
    NigglesDBC.mounts   = Niggles.updatedOptions.mounts;
    NigglesDBC.critters = Niggles.updatedOptions.critters;

    -- Save the new character lists
    NigglesDB.raidAssists = Niggles.updatedOptions.raidAssists;
    NigglesDB.moderators  = Niggles.updatedOptions.moderators;
    NigglesDB.nonRaiders  = Niggles.updatedOptions.nonRaiders;

    -- Register for events
    NigglesRegisterForEvents();

    -- Remove the updated options
    Niggles.updatedOptions = nil;

    -- Force any visible calendar event frame to update itself
    if ((CalendarCreateEventFrame) and
        (CalendarCreateEventFrame:IsShown())) then
      NigglesCalenderInvitesShow(CalendarCreateEventFrame);
    end
    if ((CalendarViewEventFrame) and
        (CalendarViewEventFrame:IsShown())) then
      NigglesCalenderInvitesShow(CalendarViewEventFrame);
    end

    -- Show/Hide the most valuable quest reward, if required
    if ((QuestFrame:IsShown()) or
        (QuestLogFrame:IsShown()) or
        (QuestLogDetailFrame:IsShown())) then
      NigglesQuestsItemInfoUpdate();
    end

    -- Force the Archaeology Summary page to update itself, if required
    if ((ArchaeologyFrameSummaryPage) and
        (ArchaeologyFrameSummaryPage:IsShown())) then
      NigglesArchaeologySummaryUpdate();
    end

    -- Force the Spell Book panel to update itself, if required
    if (SpellBookCompanionsFrame:IsShown()) then
      NigglesSpellBookCompanionUpdate();
    end
  end

  return;
end

--
-- Function to cancel changes to the options
--
function NigglesOptionsCancel()
  -- Destroy any updated options
  Niggles.updatedOptions = nil;

  return;
end

--
-- Function to change all the settings on the Niggles Options panels to their
-- default values.
--
function NigglesOptionsDefaults(frame)
  -- Destroy any updated options
  Niggles.updatedOptions = nil;

  -- Set all the settings to their default values
  for key, value in pairs(defaults) do
    NigglesDBC.settings[key] = value;
  end

  -- Clear the character lists
  NigglesDB.raidAssists = {};
  NigglesDB.moderators  = {};
  NigglesDB.nonRaiders  = {};

  -- Clear all the companions
  NigglesDBC.mounts   = {};
  NigglesDBC.critters = {};

  -- Re-initialise the settings
  NigglesOptionsRefresh();

  return;
end

--
-- Function to register/un-register for events, based on the options
--
function NigglesRegisterForEvents()
  -- Check if clicking unit buttons in the raid panel should target units
  if ((NigglesDBC.settings.RaidsTarget == true) or
      (NigglesDBC.settings.RaidsAutoPromote == true)) then
    Niggles:RegisterEvent("PARTY_MEMBERS_CHANGED");
    Niggles:RegisterEvent("RAID_ROSTER_UPDATE");
    Niggles:RegisterEvent("PLAYER_REGEN_ENABLED");
  else
    Niggles:UnregisterEvent("PARTY_MEMBERS_CHANGED");
    Niggles:UnregisterEvent("RAID_ROSTER_UPDATE");
    Niggles:UnregisterEvent("PLAYER_REGEN_ENABLED");
  end
  NigglesRaidButtonsSetTargets();

  -- Check if automatic loot settings are set
  if ((NigglesDBC.settings.RaidsAutoLootMethod == true) or
      (NigglesDBC.settings.RaidsAutoLootThold == true)) then
    Niggles:RegisterEvent("PARTY_CONVERTED_TO_RAID");
  else
    Niggles:UnregisterEvent("PARTY_CONVERTED_TO_RAID");
  end

  -- Check if warnings should be generated for pets growling
  if (NigglesDBC.settings.WarningPetGrowl == true) then
    Niggles:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
  else
    Niggles:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
  end

  -- Check if warnings should be generated for fishing rods or lances
  if ((NigglesDBC.settings.WarningRod) or
      (NigglesDBC.settings.WarningLance) or
      ((Niggles.playerClass == "HUNTER") and
       (NigglesDBC.settings.WarningNoAspect))) then
    Niggles:RegisterEvent("PLAYER_REGEN_DISABLED");
  else
    Niggles:UnregisterEvent("PLAYER_REGEN_DISABLED");
  end

  -- Check if warnings should be generated for currencies
  if (NigglesDBC.settings.WarningCurrency) then
    Niggles:RegisterEvent("CURRENCY_DISPLAY_UPDATE");
  else
    Niggles:UnregisterEvent("CURRENCY_DISPLAY_UPDATE");
  end

  -- Check if updates to the guild roster should be processed
  if (NigglesDBC.settings.CalEventsOnline) then
    Niggles:RegisterEvent("GUILD_ROSTER_UPDATE");
  else
    Niggles:UnregisterEvent("GUILD_ROSTER_UPDATE");
  end

  -- Check if updates to calendar events should be processed
  if (NigglesDBC.settings.CalEventsOnline) then
    Niggles:RegisterEvent("CALENDAR_UPDATE_INVITE_LIST");
  else
    Niggles:UnregisterEvent("CALENDAR_UPDATE_INVITE_LIST");
  end

  -- Enable/disable logging
  LoggingChat(NigglesDBC.settings.LogChat);
  LoggingCombat(NigglesDBC.settings.LogCombat);

  -- Check if some critters have been selected
  if (next(NigglesDBC.critters) ~= nil) then
    -- Recalculate the mask used to determine if critters should be summoned
    NigglesCrittersSetMask();

    -- Search for quest critters
    NigglesCrittersBagUpdate();

    -- Register for the required events
    Niggles:RegisterEvent("BAG_UPDATE");
    Niggles:RegisterEvent("UPDATE_STEALTH");
  else
    Niggles:UnregisterEvent("BAG_UPDATE");
    Niggles:UnregisterEvent("UPDATE_STEALTH");
  end

  -- Create timers, if required
  if (NigglesDBC.settings.CalEventsOnline) then
    NigglesTimerCreate("CalEventsOnline", NigglesCalEventsUpdate,
      nil, nil, 30, true);
  else
    NigglesTimerDestroy("CalEventsOnline");
  end
  if (next(NigglesDBC.critters) ~= nil) then
    NigglesTimerCreate("Critters", NigglesCritterSummon,
      nil, nil, 10, true);
  else
    NigglesTimerDestroy("Critters");
  end

  -- Set the font for the mail system
  if (NigglesDBC.settings.UiMailFont == true) then
    OpenMailBodyText:SetFontObject("QuestFont");
    SendMailBodyEditBox:SetFontObject("QuestFont");
  else
    OpenMailBodyText:SetFontObject("MailTextFontNormal");
    SendMailBodyEditBox:SetFontObject("MailTextFontNormal");
  end

  -- Show/Hide the Niggles Guild roster filter
  if (NigglesGuildRosterButton ~= nil) then
    if (NigglesDBC.settings.UiGuildRosterFilter == true) then
      NigglesGuildRosterButton:Show();
    else
      NigglesGuildRosterButton:Hide();
      NigglesGuildRosterFilterShow(false);
    end
  end

  -- Show/Hide the Map Location
  if (NigglesDBC.settings.UiMapLocation == true) then
    NigglesMapLocation:Show();
  else
    NigglesMapLocation:Hide();
  end

  -- Show/Hide the button for displaying the list of Flight Destinations
  if (NigglesDBC.settings.UiFlightList == true) then
    NigglesFlightsToggleButton:Show();
  else
    NigglesFlightsToggleButton:Hide();
  end

  -- Show/Hide the frame used to provide enhancements for the Artifact icon
  if (NigglesArtifactIcon ~= nil) then
    if (NigglesDBC.settings.ProfArchEnhIcon == true) then
      NigglesArtifactIcon:Show();
    else
      NigglesArtifactIcon:Hide();
    end
  end

  -- Enable/Disable Merchant Search
  NigglesMerchantSearchEnable();

  -- Show/Hide the buttons for toggling class colours
  if (NigglesDBC.settings.UiChatClassColor == true) then
    NigglesChatConfigChatButton:Show();
    NigglesChatConfigChannelButton:Show();
  else
    NigglesChatConfigChatButton:Hide();
    NigglesChatConfigChannelButton:Hide();
  end

  -- Hide the 'Calendar Events' frame if not enabled
  if (NigglesDBC.settings.CalEventsList ~= true) then
    HideUIPanel(NigglesCalendarEvents);
  end

  return;
end

--
-- Function to update the guild roster for any open calendar event
--
function NigglesCalEventsUpdate()
  -- Check if the guild roster needs to be updated
  if ((IsInGuild()) and
      (((CalendarCreateEventFrame) and
        (CalendarCreateEventFrame:IsShown())) or
       ((CalendarViewEventFrame) and
        (CalendarViewEventFrame:IsShown())))) then
    GuildRoster();
  end

  return;
end

--
-- Function called when the guild roster is updated
--
function NigglesGuildRosterUpdate()
  -- Local Variables
  local numMembers = GetNumGuildMembers(true);
  local memberName;
  local memberOnline;

  -- Set the last online time to an invalid value for all guild members
  for memberName, _ in pairs(Niggles.lastOnline) do
    Niggles.lastOnline[memberName] = -1;
  end

  -- Process the guild roster
  for idx = 1, numMembers do
    memberName, _, _, _, _, _, _, _, memberOnline = GetGuildRosterInfo(idx);
    if (memberName ~= nil) then
      if (memberOnline) then
        Niggles.lastOnline[memberName] = 0;
      else
        years, months, days, hours = GetGuildRosterLastOnline(idx);
        Niggles.lastOnline[memberName] =
          ((((((years or 0)*12)+(months or 0))*30.5)+
          (days or 0))*24)+(hours or 0)+0.1;
      end
    end
  end

  -- Remove any guild members that have left the guild
  for memberName, lastOnline in pairs(Niggles.lastOnline) do
    if (lastOnline < 0) then
      Niggles.lastOnline[memberName] = nil;
    end
  end

  -- Update any Calendar Event frame currently shown
  if ((CalendarCreateEventFrame) and
      (CalendarCreateEventFrame:IsShown())) then
    NigglesCalendarInvitesUpdate(CalendarCreateEventInviteListScrollFrame);
    NigglesCalendarInvitesFilterEnable();
  end
  if ((CalendarViewEventFrame) and
      (CalendarViewEventFrame:IsShown())) then
    NigglesCalendarInvitesUpdate(CalendarViewEventInviteListScrollFrame);
  end

  return;
end

--
-- Function to enable/disable frames in the Niggles Options Raids panel
--
function NigglesOptionsWarningsEnableFrames()
  -- Enable or disable frames based on the updated options
  if (Niggles.updatedOptions.settings.WarningCurrency == true) then
    UIDropDownMenu_EnableDropDown(NigglesOptionsWarningsWarningCurrencyMenu);
  else
    UIDropDownMenu_DisableDropDown(NigglesOptionsWarningsWarningCurrencyMenu);
  end

  return;
end
